objectdef autolayout
{
	/* Used for auto-detect mode. For widescreen, best layout is PiP windows at top or bottom (bottom default). */
	variable bool WideScreen=FALSE

	/* Divisors used to calculate PiP and full window sizes. Should both be identical for now. It's separated for future features. */
	variable int PiPDivisor=5
	variable int FullDivisor=5
	
	/* Our constructor handles widescreen autodetection. */
	method Initialize()
	{
		/* Normal screen ratio is 4:3, so check if our Width:Height ratio is higher to see if it's widescreen */
		if ${Display.DesktopWidth}/${Display.DesktopHeight}>1.34
			WideScreen:Set[TRUE]

		if !${Session(exists)}
		{
			PiPDivisor:Dec
			FullDivisor:Dec
		}
	}

	/* Automatically position windows along the right edge of the screen */
	method ApplyRight(int nSession)
	{
		variable int pipWidth
		variable int pipHeight
		pipHeight:Set[${Display.DesktopHeight}/${PiPDivisor}]
		pipWidth:Set[${Display.DesktopWidth}/(${FullDivisor}+1)]
		
		variable int fullWidth
		fullWidth:Set[${Display.DesktopWidth}-${pipWidth}]

		variable int RightColumn
		RightColumn:Set[${fullWidth}+${Display.DesktopX}]

		if !${Session(exists)}
		{
			if !${Script[pipsqueak](exists)}
				run pipsqueak

			variable int Count
			
			pipsqueak -slot 1 ${Display.DesktopX},${Display.DesktopY},${fullWidth},${Display.DesktopHeight}
			for (Count:Set[1] ; ${Count}<=${PiPDivisor} ; Count:Inc)
			{
				pipsqueak -slot ${Math.Calc[${Count}+1].Int} ${RightColumn},${Math.Calc[(${pipHeight}*(${Count}-1))%${Display.DesktopHeight}].Int},${pipWidth},${pipHeight}
			}
			return
		}

		if !${Script[windowsnapper](exists)}
			run windowsnapper -pos ${RightColumn},${Math.Calc[(${pipHeight}*(${nSession}-1))%${Display.DesktopHeight}].Int} -size ${pipWidth}x${pipHeight}
		else
		{
			snapset -pos ${RightColumn},${Math.Calc[(${pipHeight}*(${nSession}-1))%${Display.DesktopHeight}].Int} -size ${pipWidth}x${pipHeight}
		}
	
           
		windowpos -viewable ${Display.DesktopX},${Display.DesktopY}
		windowsize -viewable ${fullWidth}x${Display.DesktopHeight}
	}



	/* Automatically position windows along the bottom edge of the screen */
	method ApplyBottom(int nSession)
	{
		variable int pipWidth
		variable int pipHeight

		pipHeight:Set[${Display.DesktopHeight}/(${FullDivisor}+1)]	
		pipWidth:Set[${Display.DesktopWidth}/${PiPDivisor}]

		variable int fullHeight
		fullHeight:Set[${Display.DesktopHeight}-${pipHeight}]

		variable int BottomRow
		BottomRow:Set[${fullHeight}+${Display.DesktopY}]
   
		if !${Session(exists)}
		{
			if !${Script[pipsqueak](exists)}
				run pipsqueak

			variable int Count
			
			pipsqueak -slot 1 ${Display.DesktopX},${Display.DesktopY},${Display.DesktopWidth},${fullHeight}
			for (Count:Set[1] ; ${Count}<=${PiPDivisor} ; Count:Inc)
			{
				pipsqueak -slot ${Math.Calc[${Count}+1].Int} ${Math.Calc[(${pipWidth}*(${Count}-1))%${Display.DesktopWidth}].Int},${BottomRow},${pipWidth},${pipHeight}
			}
			return
		}


		if !${Script[windowsnapper](exists)}
			run windowsnapper -pos ${Math.Calc[(${pipWidth}*(${nSession}-1))%${Display.DesktopWidth}].Int},${BottomRow} -size ${pipWidth}x${pipHeight}
		else
			snapset -pos ${Math.Calc[(${pipWidth}*(${nSession}-1))%${Display.DesktopWidth}].Int},${BottomRow} -size ${pipWidth}x${pipHeight}
		windowpos -viewable ${Display.DesktopX},${Display.DesktopY}
		windowsize -viewable ${Display.DesktopWidth}x${fullHeight}
	}

	/* Automatically position windows along the left edge of the screen */
	method ApplyLeft(int nSession)
	{
		variable int pipWidth
		variable int pipHeight
		pipHeight:Set[${Display.DesktopHeight}/${PiPDivisor}]
		pipWidth:Set[${Display.DesktopWidth}/(${FullDivisor}+1)]
		
		variable int fullWidth
		fullWidth:Set[${Display.DesktopWidth}-${pipWidth}]
		
		variable int RightColumn
		RightColumn:Set[${pipWidth}+${Display.DesktopX}]
    	
		if !${Session(exists)}
		{
			if !${Script[pipsqueak](exists)}
				run pipsqueak

			variable int Count
			
			pipsqueak -slot 1 ${RightColumn},${Display.DesktopY},${fullWidth},${Display.DesktopHeight}
			for (Count:Set[1] ; ${Count}<=${PiPDivisor} ; Count:Inc)
			{
				pipsqueak -slot ${Math.Calc[${Count}+1].Int} ${Display.DesktopX},${Math.Calc[(${pipHeight}*(${Count}-1))%${Display.DesktopHeight}].Int},${pipWidth},${pipHeight}
			}
			return
		}

		if !${Script[windowsnapper](exists)}
			run windowsnapper -pos ${Display.DesktopX},${Math.Calc[(${pipHeight}*(${nSession}-1))%${Display.DesktopHeight}].Int} -size ${pipWidth}x${pipHeight}
		else
			snapset -pos ${Display.DesktopX},${Math.Calc[(${pipHeight}*(${nSession}-1))%${Display.DesktopHeight}].Int} -size ${pipWidth}x${pipHeight}

		windowpos -viewable ${RightColumn},${Display.DesktopY}
		windowsize -viewable ${fullWidth}x${Display.DesktopHeight}
	}

	/* Automatically position windows along the top edge of the screen */
	method ApplyTop(int nSession)
	{
		variable int pipWidth
		variable int pipHeight
		pipWidth:Set[${Display.DesktopWidth}/${PiPDivisor}]
		pipHeight:Set[${Display.DesktopHeight}/(${FullDivisor}+1)]
		
		variable int fullHeight
		fullHeight:Set[${Display.DesktopHeight}-${pipHeight}]

		variable int BottomRow
		BottomRow:Set[${pipHeight}+${Display.DesktopY}]
    

		if !${Session(exists)}
		{
			if !${Script[pipsqueak](exists)}
				run pipsqueak

			variable int Count
			
			pipsqueak -slot 1 ${Display.DesktopX},${BottomRow},${Display.DesktopWidth},${fullHeight}
			for (Count:Set[1] ; ${Count}<=${PiPDivisor} ; Count:Inc)
			{
				pipsqueak -slot ${Math.Calc[${Count}+1].Int} ${Math.Calc[(${pipWidth}*(${Count}-1))%${Display.DesktopWidth}].Int},${Display.DesktopY},${pipWidth},${pipHeight}
			}
			return
		}

		if !${Script[windowsnapper](exists)}
			run windowsnapper -pos ${Math.Calc[(${pipWidth}*(${nSession}-1))%${Display.DesktopWidth}].Int},${Display.DesktopY} -size ${pipWidth}x${pipHeight}
		else
			snapset -pos ${Math.Calc[(${pipWidth}*(${nSession}-1))%${Display.DesktopWidth}].Int},${Display.DesktopY} -size ${pipWidth}x${pipHeight}
		windowpos -viewable ${Display.DesktopX},${BottomRow}
		windowsize -viewable ${Display.DesktopWidth}x${fullHeight}
	}

	/* Automatically position windows along the outer right edge of the screen */
	method ApplyRightOutside(int nSession)
	{
		variable int pipWidth
		variable int pipHeight
		pipHeight:Set[${Display.DesktopHeight}/${PiPDivisor}]
		pipWidth:Set[${Display.DesktopWidth}/${PiPDivisor}]
		
		variable int RightColumn
		RightColumn:Set[${Display.DesktopX}+${Display.DesktopWidth}]

		if !${Session(exists)}
		{
			if !${Script[pipsqueak](exists)}
				run pipsqueak

			variable int Count
			
			pipsqueak -slot 1 ${Display.DesktopX},${Display.DesktopY},${Display.DesktopWidth},${Display.DesktopHeight}
			for (Count:Set[1] ; ${Count}<=${PiPDivisor} ; Count:Inc)
			{
				pipsqueak -slot ${Math.Calc[${Count}+1].Int} ${RightColumn},${Math.Calc[(${pipHeight}*(${Count}-1))%${Display.DesktopHeight}].Int},${pipWidth},${pipHeight}
			}
			return
		}

		if !${Script[windowsnapper](exists)}
			run windowsnapper -pos ${RightColumn},${Math.Calc[(${pipHeight}*(${nSession}-1))%${Display.DesktopHeight}].Int} -size ${pipWidth}x${pipHeight}
		else
		{
			snapset -pos ${RightColumn},${Math.Calc[(${pipHeight}*(${nSession}-1))%${Display.DesktopHeight}].Int} -size ${pipWidth}x${pipHeight}
		}
	
           
		windowpos -viewable ${Display.DesktopX},${Display.DesktopY}
		windowsize -viewable ${Display.DesktopWidth}x${Display.DesktopHeight}
	}

	/* Automatically position windows along the outer left edge of the screen */
	method ApplyLeftOutside(int nSession)
	{
		variable int pipWidth
		variable int pipHeight
		pipHeight:Set[${Display.DesktopHeight}/${PiPDivisor}]
		pipWidth:Set[${Display.DesktopWidth}/${PiPDivisor}]
		
		variable int RightColumn
		RightColumn:Set[${Display.DesktopX}-${pipWidth}]

		if !${Session(exists)}
		{
			if !${Script[pipsqueak](exists)}
				run pipsqueak

			variable int Count
			
			pipsqueak -slot 1 ${Display.DesktopX},${Display.DesktopY},${Display.DesktopWidth},${Display.DesktopHeight}
			for (Count:Set[1] ; ${Count}<=${PiPDivisor} ; Count:Inc)
			{
				pipsqueak -slot ${Math.Calc[${Count}+1].Int} ${RightColumn},${Math.Calc[(${pipHeight}*(${Count}-1))%${Display.DesktopHeight}].Int},${pipWidth},${pipHeight}
			}
			return
		}

		if !${Script[windowsnapper](exists)}
			run windowsnapper -pos ${RightColumn},${Math.Calc[(${pipHeight}*(${nSession}-1))%${Display.DesktopHeight}].Int} -size ${pipWidth}x${pipHeight}
		else
		{
			snapset -pos ${RightColumn},${Math.Calc[(${pipHeight}*(${nSession}-1))%${Display.DesktopHeight}].Int} -size ${pipWidth}x${pipHeight}
		}
	
           
		windowpos -viewable ${Display.DesktopX},${Display.DesktopY}
		windowsize -viewable ${Display.DesktopWidth}x${Display.DesktopHeight}
	}

	/* Automatically position windows along the outside of the bottom edge of the screen */
	method ApplyBottomOutside(int nSession)
	{
		variable int pipWidth
		variable int pipHeight
		pipWidth:Set[${Display.DesktopWidth}/${PiPDivisor}]
		pipHeight:Set[${Display.DesktopHeight}/${PiPDivisor}]
		
		variable int BottomRow
		BottomRow:Set[${Display.DesktopY}+${Display.DesktopHeight}]
   
		if !${Session(exists)}
		{
			if !${Script[pipsqueak](exists)}
				run pipsqueak

			variable int Count
			
			pipsqueak -slot 1 ${Display.DesktopX},${Display.DesktopY},${Display.DesktopWidth},${Display.DesktopHeight}
			for (Count:Set[1] ; ${Count}<=${PiPDivisor} ; Count:Inc)
			{
				pipsqueak -slot ${Math.Calc[${Count}+1].Int} ${Math.Calc[(${pipWidth}*(${Count}-1))%${Display.DesktopWidth}].Int},${BottomRow},${pipWidth},${pipHeight}
			}
			return
		}

		if !${Script[windowsnapper](exists)}
			run windowsnapper -pos ${Math.Calc[(${pipWidth}*(${nSession}-1))%${Display.DesktopWidth}].Int},${BottomRow} -size ${pipWidth}x${pipHeight}
		else
			snapset -pos ${Math.Calc[(${pipWidth}*(${nSession}-1))%${Display.DesktopWidth}].Int},${BottomRow} -size ${pipWidth}x${pipHeight}
		windowpos -viewable ${Display.DesktopX},${Display.DesktopY}
		windowsize -viewable ${Display.DesktopWidth}x${Display.DesktopHeight}
	}

	/* Automatically position windows along the outer top edge of the screen */
	method ApplyTopOutside(int nSession)
	{
		variable int pipWidth
		variable int pipHeight
		pipWidth:Set[${Display.DesktopWidth}/${PiPDivisor}]
		pipHeight:Set[${Display.DesktopHeight}/${PiPDivisor}]
		
		variable int BottomRow
		BottomRow:Set[${Display.DesktopY}-${pipHeight}]

		if !${Session(exists)}
		{
			if !${Script[pipsqueak](exists)}
				run pipsqueak

			variable int Count
			
			pipsqueak -slot 1 ${Display.DesktopX},${Display.DesktopY},${Display.DesktopWidth},${Display.DesktopHeight}
			for (Count:Set[1] ; ${Count}<=${PiPDivisor} ; Count:Inc)
			{
				pipsqueak -slot ${Math.Calc[${Count}+1].Int} ${Math.Calc[(${pipWidth}*(${Count}-1))%${Display.DesktopWidth}].Int},${BottomRow},${BottomRow},${pipWidth},${pipHeight}
			}
			return
		}

		if !${Script[windowsnapper](exists)}
			run windowsnapper -pos ${Math.Calc[(${pipWidth}*(${nSession}-1))%${Display.DesktopWidth}].Int},${BottomRow} -size ${pipWidth}x${pipHeight}
		else
			snapset -pos ${Math.Calc[(${pipWidth}*(${nSession}-1))%${Display.DesktopWidth}].Int},${BottomRow} -size ${pipWidth}x${pipHeight}
		windowpos -viewable ${Display.DesktopX},${Display.DesktopY}
		windowsize -viewable ${Display.DesktopWidth}x${Display.DesktopHeight}
	}

	/* Automatically position windows along the bottom (widescreen) or right (standard screen) edge of the screen */
	method ApplySession(int nSession)
	{
		if ${WideScreen}
			This:ApplyBottom[${nSession}]
		else
			This:ApplyRight[${nSession}]		
	}
}

variable autolayout AutoLayout

/* Main entry point.

Syntax: run autolayout
        ... If no parameters are given, uses "auto" layout, with a divisor (window count) of 5.
or
        run autolayout [#] [outer] <EDGE>
   #       If a numeric option is given, this will be used as the divisor. This would typically match the number of windows. If not given,
           defaults to 5.
   outer   If given, will be on the OUTER edge of the display, rather than making room on the inner edge. Only recommended when dragging
           a window to the other monitor will give full performance (i.e. should not be used if monitors are on different video cards, or
           even with the same video card on XP -- same video card with Vista should work fine)
   EDGE    Can be any of left, right, top, bottom, or auto. Windows will be positioned along this edge. If auto is given, the edge will
           automatically be bottom for wide screens, or right for standard screens.

*/
function main(... Params)
{
	if !${Params.Size}
	{
		/* No parameters. Use auto edge, 5 windows. */
		AutoLayout:ApplySession[${Session.Right[-2]}]
		return
	}

	variable int Count=1
	for (${Count}<=${Params.Size} ; Count:Inc)
	{
		/* Note: STRING COMPARISON to the INTEGER VALUE of the parameter, this checks if it is a number.
                         You typically do NOT want to copy this form for an example "if", it is a special case.  */
		if ${Params[${Count}].Equal[${Int["${Params[${Count}]}"]}]}
		{
			/* It's a number, treat it as both divisors */
			AutoLayout.PiPDivisor:Set["${Params[${Count}]}"]
			AutoLayout.FullDivisor:Set["${Params[${Count}]}"]

			if !${Session(exists)}
			{
				PiPDivisor:Dec
				FullDivisor:Dec
			}
		}
		else
		{
			/* Not a number, so it must be one of our desired options, and we should only have 1 of them */
			switch ${Params[${Count}]}
			{
			case outer
			case outside
				Count:Inc
				if ${Count}<=${Params.Size}
				{
					switch ${Params[${Count}]}
					{
					case left
						AutoLayout:ApplyLeftOutside[${Session.Right[-2]}]
						return
					case top
						AutoLayout:ApplyTopOutside[${Session.Right[-2]}]
						return
					case right
						AutoLayout:ApplyRightOutside[${Session.Right[-2]}]
						return
					case bottom
						AutoLayout:ApplyBottomOutside[${Session.Right[-2]}]
						return	
					}		
				}
				break
			case left
				AutoLayout:ApplyLeft[${Session.Right[-2]}]
				return
			case top
				AutoLayout:ApplyTop[${Session.Right[-2]}]
				return
			case right
				AutoLayout:ApplyRight[${Session.Right[-2]}]
				return
			case bottom
				AutoLayout:ApplyBottom[${Session.Right[-2]}]
				return
			case auto
				AutoLayout:ApplySession[${Session.Right[-2]}]
				return
			default
				echo AutoLayout: Unrecognized option ${Params[${Count}]}
				break
			}
		}
	}

}



