/* Our windowsnapper class */
objectdef windowsnapper
{
	/* Late initialization method, as opposed to using the object constructor. We want to  
	   create the object before we initialize it.
        */
	method Go()
	{
		/* Registering events before we attach ensures that the event exists, to attach to */
		LavishScript:RegisterEvent[On Deactivate]
		LavishScript:RegisterEvent[On Activate]

		LavishScript:RegisterEvent[OnMouseEnter]
		LavishScript:RegisterEvent[OnMouseExit]
		LavishScript:RegisterEvent[OnFrame]

		/* Attach our methods to the events we registered for */
		Event[On Deactivate]:AttachAtom[This:OnDeactivate]
		Event[On Activate]:AttachAtom[This:OnActivate]

		Event[OnMouseEnter]:AttachAtom[This:OnMouseEnter]
		Event[OnMouseExit]:AttachAtom[This:OnMouseExit]

		/* Spit out startup spam */
		echo \agWindowSnapper: Window will snap to ${SnapPos} with size ${SnapSize}
		echo To change snap position, enter command (e.g.) \aysnappos 123,456
		echo To change snap size, enter command (e.g.) \aysnapsize 640x480
		echo WindowSnapper also accepts -size 640x480 and -pos 123,456 as parameters, such as
		echo \ayrun windowsnapper -size 640x480 -pos 123,456
	}

	/* When Store is called, we populate these */
	variable int X
	variable int Y
	variable int Width
	variable int Height
	variable string Frame

	/* This tracks whether we're the active window or not. Without it, window size and position could
           get messy. */
	variable bool Active=TRUE

	variable bool Ignore=FALSE
	variable bool AutoToggleIgnore=TRUE

	/* TRUE when windowsnapper is in Draggable mode, for dragging to set the snap-to size and position */
	variable bool Draggable=FALSE

	variable bool RollOverEnabled=FALSE
	variable bool RolledOver=FALSE
	variable float RollOverScale=1.5
	variable int RollOverFrame=0
	variable int RollOverMaxFrame=1
	variable int RollOverFrameMod=0
	variable int RollOverMaxFrameModUp=1
	variable int RollOverMaxFrameModDown=1

	/* This is our snap-to size and position. When not the active window, we use these. */
	variable string SnapSize=320x240
	variable string SnapPos=800,0

	variable bool AlwaysOnTop=TRUE

	variable string WheelSwitcherKey=Ctrl
	variable bool WheelSwitcher=FALSE

	variable bool PiPSqueak=FALSE

	/* Grabs the X-position from SnapPos */
	member:int SnapX()
	{
		return ${Arg[1,${SnapPos}]}
	}

	/* Grabs the Y-position from SnapPos */
	member:int SnapY()
	{
		return ${Arg[2,${SnapPos}]}
	}

	/* Grabs the width from SnapSize */
	member:int SnapWidth()
	{
		return ${SnapSize.Token[1,x]}
	}

	/* Grabs the height from SnapSize */
	member:int SnapHeight()
	{
		return ${SnapSize.Token[2,x]}
	}

	/* Store the current display position and size for later use */
	method Store()
	{
		if ${Display.ViewableWidth}
		{
			Width:Set[${Display.ViewableWidth}]
			Height:Set[${Display.ViewableHeight}]
			X:Set[${Display.Window.X}]
			Y:Set[${Display.Window.Y}]
		}
		else
		{
			if !${Width}
			{
				Width:Set[${Display.Width}]
				Height:Set[${Display.Height}]
			}
		}
		Frame:Set["${Display.Window.Frame}"]
	}

	/* Restore the stored display position and size */
	method Restore()
	{
		windowcharacteristics -stealth -pos ${X},${Y} -size -viewable ${Width}x${Height} -frame ${Frame}
	}

	/* Restore the stored display position and size */
	method RestoreFull()
	{
		windowcharacteristics -pos ${X},${Y} -size -viewable ${Width}x${Height} -frame ${Frame}
	}

	/* Toggles draggable mode ... (comments ftw) */
	method ToggleDraggable()
	{
		if ${Draggable}
			This:EndDraggable
		else
			This:BeginDraggable
	}

	/* Enters draggable mode, allowing dragging to set the snap-to size and position */
	method BeginDraggable()
	{
		if ${Draggable}
			return
		Event[On Activate]:DetachAtom[This:OnActivate]
		Draggable:Set[TRUE]
		This:Store
		This:SnapTo
		windowcharacteristics -stealth -frame thick
	}

	/* Completes dragging operation, setting the final snap-to size and position and restoring the full size */
	method EndDraggable()
	{
		if !${Draggable}
			return
		Draggable:Set[FALSE]
		snapsize "${Display.Window.Width}x${Display.Window.Height}"
		snappos "${Display.Window.X},${Display.Window.Y}"
		This:RestoreFull
		Event[On Activate]:AttachAtom[This:OnActivate]
	}

	method EnableRollOver(float scale)
	{
		if ${scale}>=1
			RollOverScale:Set[${scale}]
;	variable bool RollOverEnabled=FALSE
;	variable bool RolledOver=FALSE
;	variable float RollOverScale=1.5
		if ${RollOverEnabled}
			return

		RollOverEnabled:Set[TRUE]
;		Event[OnMouseEnter]:AttachAtom[This:OnMouseEnter]
;		Event[OnMouseExit]:AttachAtom[This:OnMouseExit]
	}

	method DisableRollOver()
	{
		if !${RollOverEnabled}
			return
		RollOverEnabled:Set[FALSE]
;		Event[OnMouseEnter]:DetachAtom[This:OnMouseEnter]
;		Event[OnMouseExit]:DetachAtom[This:OnMouseExit]
	}

	method ApplyRollOverScale(float scale)
	{
		/* See if we need to re-position before we move to keep it on-screen... */		

		variable int newWidth
		variable int newHeight
		newWidth:Set[${This.SnapWidth}*${scale}]
		newHeight:Set[${This.SnapHeight}*${scale}]

;		echo newSize = ${newWidth}x${newHeight}

		variable int rightDisplayEdge
		variable int bottomDisplayEdge
		rightDisplayEdge:Set[${Display.Monitor[${CurrentMonitor}].Right}]
		bottomDisplayEdge:Set[${Display.Monitor[${CurrentMonitor}].Bottom}]

;		echo DisplayEdge = ${rightDisplayEdge},${bottomDisplayEdge}

		variable int useX
		variable int useY
		useX:Set[${This.SnapX}-((${newWidth}-${This.SnapWidth})/2)]
		useY:Set[${This.SnapY}-((${newHeight}-${This.SnapHeight})/2)]

;		echo new pos = ${useX},${useY}

		if ${useX}<=${Display.Monitor[${CurrentMonitor}].Left}
			useX:Set[${Display.Monitor[${CurrentMonitor}].Left}]
		elseif ${newWidth}+${useX}>${rightDisplayEdge}
			useX:Set[${rightDisplayEdge}-${newWidth}]	

		if ${useY}<=${Display.Monitor[${CurrentMonitor}].Top}
			useY:Set[${Display.Monitor[${CurrentMonitor}].Top}]
		elseif ${newHeight}+${useY}>${bottomDisplayEdge}
			useY:Set[${bottomDisplayEdge}-${newHeight}]	

;		echo new pos modified = ${useX},${useY}

		if ${RolledOver}
		{
			if ${AlwaysOnTop}
				windowcharacteristics -stealth -visibility -noactivate alwaysontop -pos ${useX},${useY} -size -viewable ${newWidth}x${newHeight}
			else
				windowcharacteristics -stealth -visibility -noactivate foreground -pos ${useX},${useY} -size -viewable ${newWidth}x${newHeight}

		}
		else
			windowcharacteristics -stealth -pos ${useX},${useY} -size -viewable ${newWidth}x${newHeight}

		if ${RolledOver}
		{
			Mouse.ScaleX:Set[${Display.Width}/${Display.Window.ViewableWidth}]
			Mouse.ScaleY:Set[${Display.Height}/${Display.Window.ViewableHeight}]
		}

	}

	method OnFrame()
	{
		if !${RollOverEnabled} || (!${RolledOver} && !${RollOverFrame}) || ${Active}
		{
			Event[OnFrame]:DetachAtom[This:OnFrame]
			return
		}
		if ${RolledOver}
		{
			if ${RollOverFrame}==${RollOverMaxFrame}
			{
				return
			}
			if ${RollOverFrame}
			{
				RollOverFrameMod:Inc
				if ${RollOverFrameMod}<${RollOverMaxFrameModUp}
					return
				RollOverFrameMod:Set[0]
			}
			RollOverFrame:Inc
		}
		else
		{
			if !${RollOverFrame}
				return
			if ${RollOverFrame}
			{
				RollOverFrameMod:Inc
				if ${RollOverFrameMod}<${RollOverMaxFrameModDown}
					return
				RollOverFrameMod:Set[0]
			}
			RollOverFrame:Dec
		}
		variable float scale
		scale:Set[(${RollOverFrame}*(${RollOverScale}-1)/${RollOverMaxFrame})+1]
		
/*
		echo timing ApplyRollOverScale
		execute -time \
/**/
		This:ApplyRollOverScale[${scale}]
	}

	member:uint GetMonitor()
	{
		if ${Display.Monitors}<2
			return 1

		variable uint i=1
		variable int MidX
		variable int MidY
		MidX:Set[${Display.X}+(${Display.Window.Width}/2)]
		MidY:Set[${Display.Y}+(${Display.Window.Height}/2)]
		for (${i}<=${Display.Monitors} ; i:Inc)
		{
			if ${Display.Monitor[${i}].Left}>${MidX}
				continue
			if ${Display.Monitor[${i}].Right}<${MidX}
				continue
			if ${Display.Monitor[${i}].Top}>${MidY}
				continue
			if ${Display.Monitor[${i}].Bottom}<${MidY}
				continue
			return ${i}
		}

		return 1
	}

	variable uint CurrentMonitor=1

	method OnMouseEnter()
	{	
		if ${Active}
		{
			return
		}

		CurrentMonitor:Set[${This.GetMonitor}]
;		echo OnMouseEnter
		RolledOver:Set[TRUE]
		RollOverFrameMod:Set[0]
		if ${RollOverEnabled}
			Event[OnFrame]:AttachAtom[This:OnFrame]
;		Mouse.ScaleX:Set[${Display.Width}/${Display.Window.ViewableWidth}]
;		Mouse.ScaleY:Set[${Display.Height}/${Display.Window.ViewableHeight}]
	}

	method OnMouseExit()
	{
		if !${RolledOver}
			return
		RolledOver:Set[FALSE]
		if ${Active}
		{
			Event[OnFrame]:DetachAtom[This:OnFrame]
			return
		}
		RollOverFrameMod:Set[0]
;		Mouse.ScaleX:Set[1]
;		Mouse.ScaleY:Set[1]
	}


	/* Snaps the window TO the snap-to size and position, with always on top */
	method SnapTo()
	{
		if ${AlwaysOnTop}
		{
			windowcharacteristics -stealth -frame none -size -viewable ${SnapSize} -pos ${SnapPos} -visibility -noactivate alwaysontop
		}
		else
		{
			windowcharacteristics -stealth -frame none -size -viewable ${SnapSize} -pos ${SnapPos}
		}
	}

	/* Snaps the window back to full size, and removes always on top */
	method SnapBack()
	{
		This:Restore
		windowcharacteristics -stealth -visibility foreground
		Mouse.ScaleX:Set[1]
		Mouse.ScaleY:Set[1]
	}

	/* Handler for the "On Deactivate" event, which fires when the window is no longer the active one */
	method OnDeactivate()
	{
		if ${Ignore} || ${PiPSqueak}
		{
			return
		}
		This:Deactivate
	}
	
	/* Handler for the "On Activate" event, which fires when the window is once again the active one */
	method OnActivate()
	{
		if ${Ignore}
		{
			if ${AutoToggleIgnore}
				Ignore:Set[FALSE]
			else
			{
				return
			}
		}

		This:Activate
	}

	/* Manually activate */
	method Activate()
	{
		if !${Active}
		{
			RollOverFrame:Set[0]
/*
			echo Timing SnapBack
			execute -time \
/**/
			This:SnapBack
			Active:Set[TRUE]

		}
			if ${PiPSqueak}
			{
				uplink pipsqueak -activate "${Session}"
			}
	}

	/* Manually deactivate */
	method Deactivate()
	{
		if ${Active}
		{
			RolledOver:Set[FALSE]
			if ${Draggable}
			{
				Draggable:Set[FALSE]
				Event[On Activate]:AttachAtom[This:OnActivate]
				snapsize "${Display.Window.Width}x${Display.Window.Height}"
				snappos "${Display.Window.X},${Display.Window.Y}"
			}
			else
				This:Store
/*
			echo Timing SnapTo
			execute -time \
/**/
			This:SnapTo
			Active:Set[FALSE]
		}
	}

	/* Handler for the OnMouseWheel event, which fires when mouse wheel is moved. This is for our wheel switcher functionality. */
	method OnMouseWheel(int offset)
	{
		/* If our wheel switcher key is not pressed, let's not eat the mouse wheel movement or switch */
		if !${Input.Button["${WheelSwitcherKey.Escape}"].Pressed}
			return
		/* This makes us eat the mouse wheel movement. */	 
		Context:Set[FALSE]

		/* Now depending on the direction we moved the mouse wheel, switch to either the next or previous window */
		if ${offset}<0
			uplink focus -next
		if ${offset}>0
			uplink focus -previous
	}

	/* Enables wheel switcher functionality. Pass in ONE KEY to hold down while mouse wheeling for it to work. */
	method EnableWheelSwitcher(string key)
	{
		WheelSwitcherKey:Set["${key.Escape}"]
		if ${WheelSwitcher}
			return
		WheelSwitcher:Set[TRUE]
		Event[OnMouseWheel]:AttachAtom[This:OnMouseWheel]
	}

	/* Disables wheel switcher functionality. */
	method DisableWheelSwitcher()
	{
		if !${WheelSwitcher}
			return

		WheelSwitcher:Set[FALSE]
		Event[OnMouseWheel]:DetachAtom[This:OnMouseWheel]
	}
	

}

/* Declare our windowsnapper object, called Snapper */
variable(global) windowsnapper Snapper

/* Create a command called snapsize to change the size. The command is here until the script is terminated. */
atom(global) snapsize(string size)
{
	Snapper.SnapSize:Set["${size}"]
		echo \agWindowSnapper: Window will snap to ${Snapper.SnapPos} with size ${Snapper.SnapSize}
	if !${Snapper.Active}
		squelch windowsize -rescale -viewable ${Snapper.SnapSize}
}

/* Create a command called snappos to change the position. The command is here until the script is terminated. */
atom(global) snappos(string position)
{
	Snapper.SnapPos:Set["${position}"]
		echo \agWindowSnapper: Window will snap to ${Snapper.SnapPos} with size ${Snapper.SnapSize}
	if !${Snapper.Active}
		squelch windowpos -stealth ${Snapper.SnapPos}
}

/* Create a command called snapignore to ignore the next deactivation, until reactivated (again to change your mind) */
atom(global) snapignore()
{
	if ${Snapper.Ignore:Toggle}
	{
		echo \agWindowSnapper: Ignoring ${If[${Snapper.AutoToggleIgnore},"next ",""]}deactivation
	}
	else
	{
		echo \agWindowSnapper: NOT Ignoring ${If[${Snapper.AutoToggleIgnore},"next ",""]}deactivation
	}
}

/* Create a command called snapdrag to turn on, turn off, or toggle Dragging mode */
atom(global) snapdrag(string onofftoggle)
{
	switch ${onofftoggle}
	{
	case on
		Snapper:BeginDraggable
		break
	case toggle
		Snapper:ToggleDraggable
		break
	case off
	default
		Snapper:EndDraggable
		break
	}
}

/* Create a command called snapdock to allow docking of another session's snap-to, adjacent to this session's snap-to */
atom(global) snapdock(string session, string where)
{
	variable string pos

	switch ${where}
	{
	case top
		pos:Set["${Snapper.SnapX},${Math.Calc[${Snapper.SnapY}-${Snapper.SnapHeight}]}"]
		relay "${session}" "snappos ${pos};snapsize ${Snapper.SnapSize}"		
		break
	case bottom
		pos:Set["${Snapper.SnapX},${Math.Calc[${Snapper.SnapY}+${Snapper.SnapHeight}]}"]		
		relay "${session}" "snappos ${pos};snapsize ${Snapper.SnapSize}"		
		break
	case left
		pos:Set["${Math.Calc[${Snapper.SnapX}-${Snapper.SnapWidth}]},${Snapper.SnapY}"]		
		relay "${session}" "snappos ${pos};snapsize ${Snapper.SnapSize}"		
		break
	case right
		pos:Set["${Math.Calc[${Snapper.SnapX}+${Snapper.SnapWidth}]},${Snapper.SnapY}"]		
		relay "${session}" "snappos ${pos};snapsize ${Snapper.SnapSize}"		
		break
	default
		echo Unknown dock position, try top, bottom, left or right
		break
	}
}

atom(global) snapset(... Params)
{
	/* Process parameters */
	variable int p=1
	for (${p}<=${Params.Size} ; p:Inc)
	{
		/* Switch does a string comparison, so parameter processing is easy */
		switch ${Params[${p}]}
		{
		case -alwaysontop
			Snapper.AlwaysOnTop:Set[TRUE]
			break
		case -noalwaysontop
			Snapper.AlwaysOnTop:Set[FALSE]
			break
		case -pipsqueak
			Snapper.PiPSqueak:Set[TRUE]
			break
		case -nopipsqueak
			Snapper.PiPSqueak:Set[FALSE]
			break
		case -ignore
			Snapper.Ignore:Set[TRUE]
			break
		case -noignore
			Snapper.Ignore:Set[FALSE]
			break
		case -autotoggleignore
			Snapper.AutoToggleIgnore:Set[TRUE]
			break
		case -noautotoggleignore
			Snapper.AutoToggleIgnore:Set[FALSE]
			break
		case -activate
			Snapper:Activate
			break
		case -deactivate
			Snapper:Deactivate
			break
		case -size
			/* we got the -size switch, so if there's another parameter, set Snapper.SnapSize to the new value */
			p:Inc
			if ${p}<=${Params.Size}
				snapsize "${Params[${p}]}"
			break
		case -pos
			/* we got the -pos switch, so if there's another parameter, set Snapper.SnapPos to the new value */
			p:Inc
			if ${p}<=${Params.Size}
				snappos "${Params[${p}]}" 
			break
		case -norollover
			Snapper:DisableRollOver
			break
		case -rollover
			Snapper:EnableRollOver
			break
		case -rolloverscale
			p:Inc
			if ${p}<=${Params.Size}
				Snapper:EnableRollOver["${Params[${p}]}"]
			break
		case -rolloverframes
			p:Inc
			Snapper:EnableRollOver[${Snapper.RollOverScale}]
			if ${p}<=${Params.Size} && ${Params[${p}]}
				Snapper.RollOverMaxFrame:Set["${Params[${p}]}"]
			else
				Snapper.RollOverMaxFrame:Set["1"]

			if ${Snapper.RollOverFrame}>${Snapper.RollOverMaxFrame}
				Snapper.RollOverFrame:Set[${Snapper.RollOverMaxFrame}]
			break
		case -dragkey
			p:Inc
			if ${p}<=${Params.Size}
			{
				bind snapdragtoggle "${Params[${p}].Escape}" snapdrag toggle
			}			
			break
		case -ignorekey
			p:Inc
			if ${p}<=${Params.Size}
			{
				bind snapignore "${Params[${p}].Escape}" snapignore
			}
			break
		case -wheelswitcher
			p:Inc
			if ${p}<=${Params.Size}
			{
				Snapper:EnableWheelSwitcher["${Params[${p}].Escape}"]
			}
			break
		case -nowheelswitcher
			Snapper:DisableWheelSwitcher
			break
		}
	}
}

/* Entry point for the script. All parameters passed to it will be put in an array called Params */
function main(... Params)
{
	if ${InnerSpace.Build}<4759
	{
		echo \ayThis version of WindowSnapper requires Inner Space build 4759 or later
		return
	}
	variable float RollOverScale=1.5
	variable bool RollOver=TRUE
	variable string SnapIgnoreCombo=CTRL+ALT+P
	variable string SnapDragCombo=CTRL+ALT+O

	/* Process parameters */
	variable int p=1
	for (${p}<=${Params.Size} ; p:Inc)
	{
		/* Switch does a string comparison, so parameter processing is easy */
		switch ${Params[${p}]}
		{
		case -alwaysontop
			Snapper.AlwaysOnTop:Set[TRUE]
			break
		case -noalwaysontop
			Snapper.AlwaysOnTop:Set[FALSE]
			break
		case -pipsqueak
			Snapper.PiPSqueak:Set[TRUE]
			break
		case -nopipsqueak
			Snapper.PiPSqueak:Set[FALSE]
			break
		case -ignore
			Snapper.Ignore:Set[TRUE]
			break
		case -noignore
			Snapper.Ignore:Set[FALSE]
			break
		case -autotoggleignore
			Snapper.AutoToggleIgnore:Set[TRUE]
			break
		case -noautotoggleignore
			Snapper.AutoToggleIgnore:Set[FALSE]
			break
		case -activate
			Snapper:Activate
			break
		case -deactivate
			Snapper:Deactivate
			break
		case -size
			/* we got the -size switch, so if there's another parameter, set Snapper.SnapSize to the new value */
			p:Inc
			if ${p}<=${Params.Size}
				Snapper.SnapSize:Set["${Params[${p}]}"]
			break
		case -pos
			/* we got the -pos switch, so if there's another parameter, set Snapper.SnapPos to the new value */
			p:Inc
			if ${p}<=${Params.Size}
				Snapper.SnapPos:Set["${Params[${p}]}"]
			break
		case -norollover
			RollOver:Set[FALSE]
			break
		case -rollover
			RollOver:Set[TRUE]
			break
		case -rolloverscale
			RollOver:Set[TRUE]
			p:Inc
			if ${p}<=${Params.Size}
				RollOverScale:Set["${Params[${p}]}"]
			break
		case -rolloverframes
			RollOver:Set[TRUE]
			p:Inc
			if ${p}<=${Params.Size} && ${Params[${p}]}
			{
				Snapper.RollOverMaxFrame:Set["${Params[${p}]}"]
				if ${Snapper.RollOverFrame}>${Snapper.RollOverMaxFrame}
					Snapper.RollOverFrame:Set[${Snapper.RollOverMaxFrame}]
			}
			break
		case -dragkey
			p:Inc
			if ${p}<=${Params.Size}
			{
				SnapDragCombo:Set["${Params[${p}].Escape}"]
			}			
			break
		case -ignorekey
			{
				SnapIgnoreCombo:Set["${Params[${p}].Escape}"]
			}
			break
		case -wheelswitcher
			p:Inc
			if ${p}<=${Params.Size}
			{
				Snapper:EnableWheelSwitcher["${Params[${p}].Escape}"]
			}
			break
		case -nowheelswitcher
			Snapper:DisableWheelSwitcher
			break
		}
	}

	/* Create a bind for snapignore toggling */
	bind snapignore "${SnapIgnoreCombo.Escape}" snapignore
	bind snapdragtoggle "${SnapDragCombo.Escape}" snapdrag toggle
	

	/* Parameters have been processed, let's get going. */	 
	Snapper:Go
	if ${RollOver}
		Snapper:EnableRollOver[${RollOverScale}]

	/* Add autolayout alias so people can use this command instead of running the script. same thing.*/
	alias autolayout run autolayout

	wait 10

	if !${Display.Window.IsForeground}
		Snapper:Deactivate
	
	
	/* Now just keep the script running, so our functionality doesn't go away */
	while 1
	{
		waitframe
	}
}

atom atexit()
{
	bind -delete snapignore
	bind -delete snapdragtoggle
}
